@@ -1,17 +1,19 @@
+# -*- coding: utf-8 -*-
 """
+    celery.utils.term
+    ~~~~~~~~~~~~~~~~~
 
-term utils.
+    Terminals and colors.
 
->>> c = colored(enabled=True)
->>> print(str(c.red("the quick "), c.blue("brown ", c.bold("fox ")),
-              c.magenta(c.underline("jumps over")),
-              c.yellow(" the lazy "),
-              c.green("dog ")))
+    :copyright: (c) 2009 - 2011 by Ask Solem.
+    :license: BSD, see LICENSE for more details.
 
 """
+from __future__ import absolute_import
+
 import platform
 
-from celery.utils.encoding import safe_str
+from .encoding import safe_str
 
 BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
 OP_SEQ = "\033[%dm"
@@ -24,6 +26,16 @@ IS_WINDOWS = SYSTEM == "Windows"
 
 
 class colored(object):
+    """Terminal colored text.
+
+    Example::
+        >>> c = colored(enabled=True)
+        >>> print(str(c.red("the quick "), c.blue("brown ", c.bold("fox ")),
+        ...       c.magenta(c.underline("jumps over")),
+        ...       c.yellow(" the lazy "),
+        ...       c.green("dog ")))
+
+    """
 
     def __init__(self, *s, **kwargs):
         self.s = s
@@ -41,7 +53,11 @@ class colored(object):
                       "white": self.white}
 
     def _add(self, a, b):
-        return safe_str(a) + safe_str(b)
+        if isinstance(a, unicode):
+            a = safe_str(a)
+        if isinstance(b, unicode):
+            b = safe_str(b)
+        return str(a) + str(b)
 
     def _fold_no_color(self, a, b):
         try:
@@ -55,7 +71,9 @@ class colored(object):
         return A + B
 
     def no_color(self):
-        return reduce(self._fold_no_color, self.s)
+        if self.s:
+            return reduce(self._fold_no_color, self.s)
+        return ""
 
     def embed(self):
         prefix = ""
@@ -67,7 +85,7 @@ class colored(object):
         suffix = ""
         if self.enabled:
             suffix = RESET_SEQ
-        return self.embed() + suffix
+        return safe_str(self.embed()) + suffix
 
     def node(self, s, op):
         return self.__class__(enabled=self.enabled, op=op, *s)
@@ -139,4 +157,4 @@ class colored(object):
         return self.node(s or [""], RESET_SEQ)
 
     def __add__(self, other):
-        return safe_str(self) + safe_str(other)
+        return str(self) + str(other)
