@@ -1,11 +1,14 @@
-from __future__ import absolute_import
-from __future__ import with_statement
-
+# -*- coding: utf-8 -*-
 """
+    celery.worker.consumer
+    ~~~~~~~~~~~~~~~~~~~~~~
 
-This module contains the component responsible for consuming messages
-from the broker, processing the messages and keeping the broker connections
-up and running.
+    This module contains the component responsible for consuming messages
+    from the broker, processing the messages and keeping the broker connections
+    up and running.
+
+    :copyright: (c) 2009 - 2011 by Ask Solem.
+    :license: BSD, see LICENSE for more details.
 
 
 * :meth:`~Consumer.start` is an infinite loop, which only iterates
@@ -70,22 +73,25 @@ up and running.
   early, *then* close the connection.
 
 """
+from __future__ import absolute_import
+from __future__ import with_statement
+
 import socket
 import sys
 import threading
 import traceback
 import warnings
 
-from celery.app import app_or_default
-from celery.datastructures import AttributeDict
-from celery.exceptions import NotRegistered
-from celery.utils import noop
-from celery.utils import timer2
-from celery.utils.encoding import safe_repr
-from celery.worker import state
-from celery.worker.job import TaskRequest, InvalidTaskError
-from celery.worker.control.registry import Panel
-from celery.worker.heartbeat import Heart
+from ..app import app_or_default
+from ..datastructures import AttributeDict
+from ..exceptions import NotRegistered
+from ..utils import noop
+from ..utils import timer2
+from ..utils.encoding import safe_repr
+from . import state
+from .job import TaskRequest, InvalidTaskError
+from .control.registry import Panel
+from .heartbeat import Heart
 
 RUN = 0x1
 CLOSE = 0x2
@@ -177,11 +183,10 @@ class QoS(object):
         if pcount != self.prev:
             new_value = pcount
             if pcount > PREFETCH_COUNT_MAX:
-                self.logger.warning(
-                    "QoS: Disabled: prefetch_count exceeds %r" % (
-                        PREFETCH_COUNT_MAX, ))
+                self.logger.warning("QoS: Disabled: prefetch_count exceeds %r",
+                                    PREFETCH_COUNT_MAX)
                 new_value = 0
-            self.logger.debug("basic.qos: prefetch_count->%s" % new_value)
+            self.logger.debug("basic.qos: prefetch_count->%s", new_value)
             self.consumer.qos(prefetch_count=new_value)
             self.prev = pcount
         return pcount
@@ -226,7 +231,7 @@ class Consumer(object):
 
     #: The thread that sends event heartbeats at regular intervals.
     #: The heartbeats are used by monitors to detect that a worker
-    #: went off-line/disappeared.
+    #: went offline/disappeared.
     heart = None
 
     #: The logger instance to use.  Defaults to the default Celery logger.
@@ -243,6 +248,8 @@ class Consumer(object):
 
     #: The process mailbox (kombu pidbox node).
     pidbox_node = None
+    _pidbox_node_shutdown = None   # used for greenlets
+    _pidbox_node_stopped = None    # used for greenlets
 
     #: The current worker pool instance.
     pool = None
@@ -334,7 +341,7 @@ class Consumer(object):
         if task.revoked():
             return
 
-        self.logger.info("Got task from broker: %s" % (task.shortinfo(), ))
+        self.logger.info("Got task from broker: %s", task.shortinfo())
 
         if self.event_dispatcher.enabled:
             self.event_dispatcher.send("task-received", uuid=task.task_id,
@@ -348,8 +355,8 @@ class Consumer(object):
                 eta = timer2.to_timestamp(task.eta)
             except OverflowError, exc:
                 self.logger.error(
-                    "Couldn't convert eta %s to time stamp: %r. Task: %r" % (
-                        task.eta, exc, task.info(safe=True)),
+                    "Couldn't convert eta %s to timestamp: %r. Task: %r",
+                    task.eta, exc, task.info(safe=True),
                     exc_info=sys.exc_info())
                 task.acknowledge()
             else:
@@ -365,11 +372,11 @@ class Consumer(object):
         try:
             self.pidbox_node.handle_message(body, message)
         except KeyError, exc:
-            self.logger.error("No such control command: %s" % exc)
+            self.logger.error("No such control command: %s", exc)
         except Exception, exc:
             self.logger.error(
-                "Error occurred while handling control command: %r\n%r" % (
-                    exc, traceback.format_exc()), exc_info=sys.exc_info())
+                "Error occurred while handling control command: %r\n%r",
+                    exc, traceback.format_exc(), exc_info=sys.exc_info())
             self.reset_pidbox_node()
 
     def apply_eta_task(self, task):
@@ -398,9 +405,9 @@ class Consumer(object):
                 message.ack()
             except self.connection_errors + (AttributeError, ), exc:
                 self.logger.critical(
-                    "Couldn't ack %r: %s reason:%r" % (
+                    "Couldn't ack %r: %s reason:%r",
                         message.delivery_tag,
-                        self._message_report(body, message), exc))
+                        self._message_report(body, message), exc)
 
         try:
             body["task"]
@@ -420,12 +427,12 @@ class Consumer(object):
                                             eventer=self.event_dispatcher)
 
         except NotRegistered, exc:
-            self.logger.error(UNKNOWN_TASK_ERROR % (
-                    exc, safe_repr(body)), exc_info=sys.exc_info())
+            self.logger.error(UNKNOWN_TASK_ERROR, exc, safe_repr(body),
+                              exc_info=sys.exc_info())
             ack()
         except InvalidTaskError, exc:
-            self.logger.error(INVALID_TASK_ERROR % (
-                    str(exc), safe_repr(body)), exc_info=sys.exc_info())
+            self.logger.error(INVALID_TASK_ERROR, str(exc), safe_repr(body),
+                              exc_info=sys.exc_info())
             ack()
         else:
             self.on_task(task)
@@ -442,19 +449,21 @@ class Consumer(object):
 
     def close_connection(self):
         """Closes the current broker connection and all open channels."""
+
+        # We must set self.connection to None here, so
+        # that the green pidbox thread exits.
+        connection, self.connection = self.connection, None
+
         if self.task_consumer:
             self._debug("Closing consumer channel...")
             self.task_consumer = \
                     self.maybe_conn_error(self.task_consumer.close)
 
-        if self.broadcast_consumer:
-            self._debug("Closing broadcast channel...")
-            self.broadcast_consumer = \
-                self.maybe_conn_error(self.broadcast_consumer.channel.close)
+        self.stop_pidbox_node()
 
-        if self.connection:
+        if connection:
             self._debug("Closing broker connection...")
-            self.connection = self.maybe_conn_error(self.connection.close)
+            self.maybe_conn_error(connection.close)
 
     def stop_consumers(self, close_connection=True):
         """Stop consuming tasks and broadcast commands, also stops
@@ -500,13 +509,14 @@ class Consumer(object):
 
         """
         self.logger.critical(
-            "Can't decode message body: %r (type:%r encoding:%r raw:%r')" % (
+            "Can't decode message body: %r (type:%r encoding:%r raw:%r')",
                     exc, message.content_type, message.content_encoding,
-                    safe_repr(message.body)))
+                    safe_repr(message.body))
         message.ack()
 
     def reset_pidbox_node(self):
         """Sets up the process mailbox."""
+        self.stop_pidbox_node()
         # close previously opened channel if any.
         if self.pidbox_node.channel:
             try:
@@ -521,20 +531,37 @@ class Consumer(object):
                                         callback=self.on_control)
         self.broadcast_consumer.consume()
 
+    def stop_pidbox_node(self):
+        if self._pidbox_node_stopped:
+            self._pidbox_node_shutdown.set()
+            self._debug("Waiting for broadcast thread to shutdown...")
+            self._pidbox_node_stopped.wait()
+            self._pidbox_node_stopped = self._pidbox_node_shutdown = None
+        elif self.broadcast_consumer:
+            self._debug("Closing broadcast channel...")
+            self.broadcast_consumer = \
+                self.maybe_conn_error(self.broadcast_consumer.channel.close)
+
     def _green_pidbox_node(self):
         """Sets up the process mailbox when running in a greenlet
         environment."""
-        conn = self._open_connection()
-        self.pidbox_node.channel = conn.channel()
-        self.broadcast_consumer = self.pidbox_node.listen(
-                                        callback=self.on_control)
-        self.broadcast_consumer.consume()
-
+        # THIS CODE IS TERRIBLE
+        # Luckily work has already started rewriting the Consumer for 3.0.
+        self._pidbox_node_shutdown = threading.Event()
+        self._pidbox_node_stopped = threading.Event()
         try:
-            while self.connection:  # main connection still open?
-                conn.drain_events()
+            with self._open_connection() as conn:
+                self.pidbox_node.channel = conn.default_channel
+                self.broadcast_consumer = self.pidbox_node.listen(
+                                            callback=self.on_control)
+                with self.broadcast_consumer:
+                    while not self._pidbox_node_shutdown.isSet():
+                        try:
+                            conn.drain_events(timeout=1.0)
+                        except socket.timeout:
+                            pass
         finally:
-            conn.close()
+            self._pidbox_node_stopped.set()
 
     def reset_connection(self):
         """Re-establish the broker connection and set up consumers,
@@ -600,8 +627,8 @@ class Consumer(object):
         # Callback called for each retry while the connection
         # can't be established.
         def _error_handler(exc, interval):
-            self.logger.error("Consumer: Connection Error: %s. " % exc
-                            + "Trying again in %d seconds..." % interval)
+            self.logger.error("Consumer: Connection Error: %s. "
+                              "Trying again in %d seconds...", exc, interval)
 
         # remember that the connection is lazy, it won't establish
         # until it's needed.
@@ -643,4 +670,4 @@ class Consumer(object):
         return {"broker": conninfo, "prefetch_count": self.qos.value}
 
     def _debug(self, msg, **kwargs):
-        self.logger.debug("Consumer: %s" % (msg, ), **kwargs)
+        self.logger.debug("Consumer: %s", msg, **kwargs)
